// Code for HackerBox 0081 LET'S MAKE CONTACT Kit
//
// Requires these two libraries:
//
// SSD1306
//    Avaible from Tools > Manage Libraries
//    Final and install the one from Alexey Dynda
//
// PS2Keyboard
//    Avaible from Tools > Manage Libraries
//    Final and install the one from Paul Stoffregen
//

#include "ssd1306.h"
#include <PS2Keyboard.h>
#include <SoftwareSerial.h>

#define BUTTON    7
#define LED_RED   A0
#define LED_GRN   A1
#define LED_BLU   A2
#define BUZZER    A3
#define LASER     6
#define ISO203    2
#define KBD_DAT   8
#define KBD_CLK   3
#define RS485_DI  4
#define RS485_DE  5
#define RS485_RO  10
#define RS485_nRE 9

#define Tbit      1000  //Laser TX bit period in microsecs

int oled_x=0, oled_y=16;
int laserRXready;

PS2Keyboard keyboard;
SoftwareSerial mySerial (RS485_RO, RS485_DI);

void setup()
{
  pinMode(BUTTON, INPUT_PULLUP);
  pinMode(LED_RED, OUTPUT);
  pinMode(LED_BLU, OUTPUT);
  pinMode(LED_GRN, OUTPUT);
  pinMode(BUZZER, OUTPUT);
  pinMode(LASER, OUTPUT);
  pinMode(ISO203, INPUT);

  digitalWrite(LASER, LOW);  //Turn laser OFF to start

  ssd1306_setFixedFont(ssd1306xled_font6x8);
  ssd1306_128x64_i2c_init();
  ssd1306_clearScreen();

  //set LED to Purple
  digitalWrite(LED_RED, HIGH);
  digitalWrite(LED_BLU, HIGH);
  digitalWrite(LED_GRN, LOW);

  Serial.begin(9600);
  keyboard.begin(KBD_DAT, KBD_CLK);
  
  //setup RS-485 interface
  pinMode(RS485_DE, OUTPUT);
  pinMode(RS485_nRE, OUTPUT);
  digitalWrite(RS485_DE, LOW);  //LOW for RX, HIGH for TX
  digitalWrite(RS485_nRE, LOW); //LOW for RX, HIGH for TX
  pinMode(RS485_RO, INPUT);     //SoftwareSerial RX pin
  pinMode(RS485_DI, OUTPUT);    //SoftwareSerial TX pin
  mySerial.begin(9600);
  
  //splash initial display
  ssd1306_negativeMode();
  ssd1306_printFixed(0,  25, "   Press to Select   ", STYLE_NORMAL);
  ssd1306_positiveMode();
  delay(500);

  //set up menu display
  ssd1306_clearScreen();
  ssd1306_printFixed(0,  8, "  TX Morse on Laser  ", STYLE_NORMAL);
  ssd1306_printFixed(0, 16, "  RX Morse on Laser  ", STYLE_NORMAL);
  ssd1306_printFixed(0, 24, "  TX Text on Laser   ", STYLE_NORMAL);
  ssd1306_printFixed(0, 32, "  RX Text on Laser   ", STYLE_NORMAL);
  ssd1306_printFixed(0, 40, "  TX Text on RS-485  ", STYLE_NORMAL);
  ssd1306_printFixed(0, 48, "  RX Text on RS-485  ", STYLE_NORMAL);
}


void loop()
{
  ssd1306_negativeMode();
  ssd1306_printFixed(0,  8, "  TX Morse on Laser  ", STYLE_NORMAL);
  ssd1306_positiveMode();
  for(int c=0; c<100; c++) {
    if(digitalRead(BUTTON)==0) {
      TX_Laser_Morse();
    }
    delay(10);
  }
  ssd1306_printFixed(0,  8, "  TX Morse on Laser  ", STYLE_NORMAL);

  ssd1306_negativeMode();
  ssd1306_printFixed(0,  16, "  RX Morse on Laser  ", STYLE_NORMAL);
  ssd1306_positiveMode();
  for(int c=0; c<100; c++) {
    if(digitalRead(BUTTON)==0) {
      RX_Laser_Morse();
    }
    delay(10);
  }
  ssd1306_printFixed(0,  16, "  RX Morse on Laser  ", STYLE_NORMAL);

  ssd1306_negativeMode();
  ssd1306_printFixed(0,  24, "  TX Text on Laser   ", STYLE_NORMAL);
  ssd1306_positiveMode();
  for(int c=0; c<100; c++) {
    if(digitalRead(BUTTON)==0) {
      TX_Laser_Text();
    }
    delay(10);
  }
  ssd1306_printFixed(0,  24, "  TX Text on Laser   ", STYLE_NORMAL);

  ssd1306_negativeMode();
  ssd1306_printFixed(0,  32, "  RX Text on Laser   ", STYLE_NORMAL);
  ssd1306_positiveMode();
  for(int c=0; c<100; c++) {
    if(digitalRead(BUTTON)==0) {
      RX_Laser_Text(); 
    }
    delay(10);
  }
  ssd1306_printFixed(0,  32, "  RX Text on Laser   ", STYLE_NORMAL);

  ssd1306_negativeMode();
  ssd1306_printFixed(0,  40, "  TX Text on RS-485  ", STYLE_NORMAL);
  ssd1306_positiveMode();
  for(int c=0; c<100; c++) {
    if(digitalRead(BUTTON)==0) {
      TX_485_Text();
    }
    delay(10);
  }
  ssd1306_printFixed(0,  40, "  TX Text on RS-485  ", STYLE_NORMAL);

  ssd1306_negativeMode();
  ssd1306_printFixed(0,  48, "  RX Text on RS-485  ", STYLE_NORMAL);
  ssd1306_positiveMode();
  for(int c=0; c<100; c++) {
    if(digitalRead(BUTTON)==0) {
      RX_485_Text();
    }
    delay(10);
  }
  ssd1306_printFixed(0,  48, "  RX Text on RS-485  ", STYLE_NORMAL);
}

void blink_green(int c){
  digitalWrite(LED_RED, LOW);
  digitalWrite(LED_BLU, LOW);
  for (int i=0; i<=c; i++){
    digitalWrite(LED_GRN, HIGH);
    delay(100);
    digitalWrite(LED_GRN, LOW);
    delay(100);
  }
}

void TX_Laser_Morse() {
  //blink LED green and show mode on display
  blink_green(4);
  ssd1306_clearScreen();
  ssd1306_negativeMode();
  ssd1306_printFixed(0,  8, "  TX Morse on Laser  ", STYLE_NORMAL);
  ssd1306_positiveMode();
  //delay(200);

  // when button is pressed: make sound, fire laser
  while(1){
    if(digitalRead(BUTTON)==0) {
      tone(BUZZER, 2000);
      digitalWrite(LASER, HIGH);
    } else {
      noTone(BUZZER);
      digitalWrite(LASER, LOW);
    }
  }
}

void RX_Laser_Morse() {
  //blink LED green and show mode on display
  blink_green(4);
  ssd1306_clearScreen();
  ssd1306_negativeMode();
  ssd1306_printFixed(0,  8, "  RX Morse on Laser  ", STYLE_NORMAL);
  ssd1306_positiveMode();
  delay(300);

  // when laser detected: make sound, glow LED Blue
  while(1){
    if(digitalRead(ISO203)==0){
      tone(BUZZER, 2000);
      digitalWrite(LED_BLU, HIGH);
    } else {
      noTone(BUZZER);
      digitalWrite(LED_BLU, LOW);
    }
  }
}

void TX_Laser_Text(){
  //blink LED green and show mode on display
  blink_green(4);
  ssd1306_clearScreen();
  ssd1306_negativeMode();
  ssd1306_printFixed(0,  8, "  TX Text on Laser   ", STYLE_NORMAL);
  ssd1306_positiveMode();
  delay(300);

  //Turn Laser on to allow for aiming
  digitalWrite(LASER, HIGH);

  while(1){
    // input from serial and keyboard (so do not connect both)
    char c;
    if (keyboard.available()) {
      c = keyboard.read();
      writeOLED(c);
      laserSend(c);
    }
    if (Serial.available()) {
      c = Serial.read();
      writeOLED(c);
      laserSend(c);
    }
  }
}

void RX_Laser_Text(){
  char c;
  
  //blink LED green and show mode on display
  blink_green(4);
  ssd1306_clearScreen();
  ssd1306_negativeMode();
  ssd1306_printFixed(0,  8, "  RX Text on Laser   ", STYLE_NORMAL);
  ssd1306_positiveMode();
  delay(300);

  while(1){
    // Input from laser receiver, and output to serial port and OLED
    c = laserRead();
    Serial.print(c);
    writeOLED(c);
  }
}

void TX_485_Text(){
  //blink LED green and show mode on display
  blink_green(4);
  ssd1306_clearScreen();
  ssd1306_negativeMode();
  ssd1306_printFixed(0,  8, "  TX Text on RS-485  ", STYLE_NORMAL);
  ssd1306_positiveMode();
  delay(300);

  //switch 485 Chip control lines to TX mode (RX in setup)
  digitalWrite(RS485_DE, HIGH);  //LOW for RX, HIGH for TX
  digitalWrite(RS485_nRE, HIGH); //LOW for RX, HIGH for TX

  while(1){
    // Input from keyboard, and output to RS485 and OLED
    char c;
    if (keyboard.available()) {
      c = keyboard.read();
      mySerial.print(c); 
      writeOLED(c);
    }
    // Input from serial port, and output to RS485 and OLED
    if (Serial.available()) {
      c = Serial.read();
      mySerial.print(c);
      writeOLED(c);
    }
  }
}

void RX_485_Text(){
  char c;
  //blink LED green and show mode on display
  blink_green(4);
  ssd1306_clearScreen();
  ssd1306_negativeMode();
  ssd1306_printFixed(0,  8, "  RX Text on RS-485  ", STYLE_NORMAL);
  ssd1306_positiveMode();
  delay(300);

  while(1){
    // Input from RS-485 chip, and output to serial port and OLED
    if(mySerial.available()){
      c = mySerial.read();
      Serial.print(c);
      writeOLED(c);
    }
  }
}

void writeOLED(char c){
  char s[3];
  s[0]=c;
  s[1]=0;
  ssd1306_printFixed(oled_x, oled_y, s, STYLE_NORMAL);
  oled_x+=6;
  if(oled_x>125){
    oled_x=0;
    oled_y+=8;
    if(oled_y>48){
      oled_y=16;
    }
  }
}

void laserSend(char c){
  //Laser ON for Tbit
  digitalWrite(LASER, HIGH);
  delayMicroseconds(Tbit);
  //Laser OFF for Tbit
  digitalWrite(LASER, LOW);
  delayMicroseconds(Tbit);
  //Laser ON for Tbit
  digitalWrite(LASER, HIGH);
  delayMicroseconds(Tbit);
  
  //Pulse out 8 bits, each for Tbit, high order bit first
  for(int i=7;i>=0;i--){
    if (c&(1<<i))
      digitalWrite(LASER, HIGH);
    else
      digitalWrite(LASER, LOW);
    delayMicroseconds(Tbit);
  }

  //Laser OFF for Tbit
  digitalWrite(LASER, LOW);
  delayMicroseconds(Tbit);
  //Laser ON for Tbit
  digitalWrite(LASER, HIGH);
  delayMicroseconds(Tbit);
  //Leave laser on
}

char laserRead(){
    char c;
  int Thalf = Tbit/2;

  //A valid byte frame is:
  //   2 start bits LOW_HIGH
  //   8 payload bits
  //   2 stop bits LOW_HIGH

  //We do not RX 12 bits and check them, because we don't know where
  //they start.  Instead, we fall back to start for any framing error.
  
  //just sit in this loop until a valid frame is received and return it
  while(1){  
    c=0;  // clear received byte to zero

    //Turn OFF laser detected LED
    digitalWrite(LED_RED, LOW);
    digitalWrite(LED_BLU, LOW);
    digitalWrite(LED_GRN, LOW);

    //Wait for laser detected
    while(digitalRead(ISO203));  //detector is INVERSE LOGIC

    //Turn ON laser detected LED
    digitalWrite(LED_RED, HIGH);

    //Wait for laser transition to low (detector high)
    //   This might be the leading edge of a Start Bit 1
    while(digitalRead(ISO203)==0);  //detector is INVERSE LOGIC

    //wait to middle of period to verify still low
    //   This is possibly the middle of Start Bit 1
    //   but the inital edge detection might have been noise
    delayMicroseconds(Thalf);

    //if laser still low, OK keep processing
    //   look more likely that we are in the middle of Start Bit 1
    if (digitalRead(ISO203)) {

      //wait one bit period
      delayMicroseconds(Tbit);

      //if laser is now high, OK keep processing  (Start Bit 2)
      if (digitalRead(ISO203)==0){

        //wait one bit period (to middle of first Data Payload Bit)
        delayMicroseconds(Tbit);

        //now read 8 bit, 1 each period, high order bit first
        for(int i=7;i>=0;i--){
          c=c<<1; //shift bits already received up by one bit
          if (digitalRead(ISO203)==0)
            c=c+1;  //make lowest bit HIGH if laser is ON
          delayMicroseconds(Tbit);  //wait to next bit period
        }

        //if laser is low, OK keep processing (Stop Bit 1)
        if (digitalRead(ISO203)) {

          //wait one bit period
          delayMicroseconds(Tbit);

          //if laser is now high, OK keep processing  (Stop Bit 2)
          if (digitalRead(ISO203)==0){

            //if we get to this point, both start bits (01) and
            //  both stop bits (01) were OK, so return the 8 bits
            //  from between them as the received byte
            return(c);

            //otherwise, fall back to start and keep looking
          }
        }
      }
    }
  }
}
